import sys

sys.path.append('D:\code\python\component-template\\template')

from streamlit_rightly_component import rightly_component, set_debug, init_component, modal
import streamlit as st

set_debug()

res = st.button('点击展示浮层')
st.write(res)

# if st.button('点击展示浮层2') is True:
#   print('2222')

value = rightly_component(component_name='ClickDemo', data={"name": "hello", "value": 1})
st.write(value)

# print(ctx.session_state._state)


from streamlit_rightly_component import rightly_component, modal
import streamlit as st

def render_dagre():
    nodes = []
    edges = []

    nodes.append({
        "id": "Test1",
        "label": "Test1" 
    })

    nodes.append({
        "id": "Test2",
        "label": "Test2" 
    })

    nodes.append({
        "id": "Test3",
        "label": "Test3" 
    })

    edges.append({
        "source": "Test1",
        "target": "Test2", 
    })

    edges.append({
        "source": "Test1",
        "target": "Test3", 
    })

    rightly_component(component_name='dagre', data={ "nodes": nodes, "edges": edges })

def render_click_demo():
    # 渲染样式有问题
    st.subheader("Component with constant args")
    num_clicks = rightly_component(component_name="ClickDemo", data={'name': 'click-demo'},  default=0)
    st.markdown("You've clicked %s times!" % int(num_clicks))

def render_modal():
    open = st.button('点击展示浮层')
    st.button('点击展示浮层2')

    if 'open_modal' in st.session_state:
        open = st.session_state['open_modal']

    if open:
      st.session_state['open_modal'] = True

      def close_fn():
        del st.session_state['open_modal']

      with modal(close_fn=close_fn) as container:
          container.markdown('这是一个浮动')
          with container:
            col1, col2 = st.columns(2)
            with col1:
              render_click_demo()
            with col2:
              render_dagre()

render_modal()



from streamlit_rightly_component import rightly_component, set_debug, instance
import pandas as pd
import streamlit as st

set_debug(False, 'http://127.0.0.1:3001')

d = {'id': [1, 2], 'value': [3, 4]}
if 'df' in st.session_state:
  df = pd.DataFrame(data=st.session_state['df'])
else:
  df = pd.DataFrame(data=d)
# df = pd.DataFrame(data=d)

result = rightly_component(component_name="LuckSheet", data={
  "data": df.to_json(),
})

rightly_component(component_name="LuckSheet", data={
  "data": df.to_json(),
}, key='sheet2')

if result and result['action'] == 'onSave':
  df2 = pd.DataFrame(data=result['data']['data'])
  st.session_state['df'] = result['data']['data']


import os
import streamlit.components.v1 as components
import streamlit as st

class RightlyComponent:
    def __init__(self):
        self._RELEASE = True
        self.url = 'http://localhost:3001'
        self._component_func = self.get_component_func()
    
    def set_debug(self, release = False, url = 'http://localhost:3001'):
        self._RELEASE = release
        self.url = url
        self._component_func = self.get_component_func()

    def get_component_func(self):
        if not self._RELEASE:
            _component_func = components.declare_component(
                # We give the component a simple, descriptive name ("my_component"
                # does not fit this bill, so please choose something better for your
                # own component :)
                "rightly_component",
                # Pass `url` here to tell Streamlit that the component will be served
                # by the local dev server that you run via `npm run start`.
                # (This is useful while your component is in development.)
                url=self.url,
            )
        else:
            # When we're distributing a production version of the component, we'll
            # replace the `url` param with `path`, and point it to to the component's
            # build directory:
            parent_dir = os.path.dirname(os.path.abspath(__file__))
            build_dir = os.path.join(parent_dir, "frontend/build")
            _component_func = components.declare_component("rightly_component", path=build_dir)
        return _component_func

    def init_event(self):
        components.html('''
            <script>
                // hide-iframe
                let listerens = [];
                window.parent.addEventListener('message', (event) => {
                    const { type } = event.data;
                    console.log('[event]', event);
                    if (type === 'component-message-dispatch') {
                        // 广播
                        listerens.forEach(listeren => listeren.postMessage(event.data, '*'))
                    }
                    if (type === 'component-message-listeren') {
                        listerens.push(event.source);
                    }
                });
                const iframes = window.parent.document.getElementsByTagName('iframe');
                Array.from(iframes).forEach(iframe => {
                    const srcdoc = iframe.srcdoc;
                    if (srcdoc.includes('hide-iframe')) {
                        iframe.parentNode.style.display = 'none';
                    }
                })
            </script>
        ''')

    def init_style(self):
        components.html('''
            <script>
                // hide-iframe
                const d = window.parent.document;
                d.getElementsByClassName('main')[0].style.overflow = 'hidden';
                d.getElementsByTagName('header')[0].style.display = 'none';
                d.getElementsByTagName('footer')[0].style.display = 'none';
                d.body.style.height = '100%';
                d.body.parentElement.style.height = '100%';
                const container = d.getElementsByClassName('block-container')[0];
                container.style.maxWidth = '100%';
                container.style.padding = '0';
                
                const iframes = window.parent.document.getElementsByTagName('iframe');
                Array.from(iframes).forEach(iframe => {
                    const srcdoc = iframe.srcdoc;
                    if (srcdoc.includes('hide-iframe')) {
                        iframe.parentNode.style.display = 'none';
                    }
                })

                window.parent.addEventListener('message', (event) => {
                    const { type } = event.data;
                    if (type === 'get_client_height') {
                        // 广播
                        event.source.postMessage({
                            type: 'set_height',
                            data: window.parent.document.body.clientHeight,
                        }, '*')
                    }
                    if (type === 'get_iframe') {
                        const iframes = window.parent.document.getElementsByTagName('iframe');
                        const filter = Array.from(iframes).filter(iframe => {
                            return iframe.contentWindow == event.source
                        });
                        if (filter[0]) {
                            const doc = filter[0];
                            const fn = new Function('doc', event.data.data);
                            fn(doc);
                        }
                    }
                });
            </script>
        ''')

    def rightly_component(self, component_name, data = {}, default = {}, key = None, events = None, once=False):
        component_value = self._component_func(componentName=component_name, data=data, key=key or component_name, default=default, events=events)
        # We could modify the value returned from the component if we wanted.
        # There's no need to do this in our simple example - but it's an option.
        if once is True and 'time' in component_value:
            # 只返回一次结果
            t = st.session_state.get(f'_component_value_{key}', None)
            if t and t == component_value['time']:
                return None
            else:
                st.session_state[f'_component_value_{key}'] = component_value["time"]
                return component_value

        return component_value


instance = RightlyComponent()

def rightly_component(component_name, data = {}, default = None, key = None, events=None, once=False):
    """ 渲染自定义组件

    Args:
        component_name (string): 组件名称
        data (dict, optional): 传给组件的参数. Defaults to {}.
        default (dict, optional): 默认返回值. Defaults to None.
        key (string, optional): 别名，默认与组件名称一致. Defaults to None.
        events (dict, optional): 事件订阅，组件间数据传递. Defaults to None.
        once (bool, optional): 只触发一次返回通知，适用于提交等操作. Defaults to False.

    Returns:
        dict: 组件操作返回结果
    """
    return instance.rightly_component(component_name, data, default, key, events, once)

def set_debug(release = False, url = 'http://localhost:3001'):
    return instance.set_debug(release, url)

def init_component():
    instance.init_event()
    instance.init_style()

class modal:
    def __init__(self, close_fn = None):
        self.action = None
        self.container = None
        self.content_container = None
        self.close_fn = close_fn

    def __enter__(self):
        self.container = st.container()
        self.container.markdown('<div id="modal-mask"></div>', unsafe_allow_html=True)
        with self.container:
          components.html('''
              <script>
                  var d = window.parent.document;
                  var b = d.getElementById('modal-mask');
                  var c = b.parentNode.parentNode.parentNode.parentNode;
                  c.id += 'modal';
                  console.log(c.className)
              </script>
          ''')
        self.container.write('''<style>
            .appview-container > section > div > div > div[data-testid="stVerticalBlock"] > div > div[data-testid="stVerticalBlock"] {
                display: none;
            }
            
            #modal {
                display: block;
                position: absolute;
                left: 0;
                top: 0;
                z-index: 900;
                width: 100%;
                height: 100%;
            }

            #modal > div:nth-child(-n + 3) {
                display: none;
            }

            #modal > div:first-child {
                display: block;
                width: 100%;
                height: 100%;
                background: rgba(333, 333, 333, 0.5);
            }

            #modal > .element-container:last-child {
                display: block;
                position: absolute;
                left: calc(50% + 200px);
                top: 50%;
                z-index: 2;
            }

            #modal > div:nth-child(4) > div > div {
              width: 100%;
            }

            #modal > div:nth-child(4) > div > div > div {
              width: 100%;
            }

            #modal > div:nth-child(4) > div > div > iframe {
              width: 100%;
            }

            #modal > div:nth-child(4) > div > div:last-child {
              position: absolute;
              left: calc(100% - 42px);
              top: 6px;
            }

            #modal > div:nth-child(4) > div {
                display: block;
                position: absolute;
                width: 100%;
                left: 0;
                height: 80%;
                bottom: 0;
                z-index: 1;
                background: #fff;
                padding: 20px;
                overflow-x: hidden;
                overflow-y: auto;
            }
            
            </style>''', unsafe_allow_html=True)
        self.content_container = self.container.container()
        return self.content_container

    def __exit__(self, exc_type, exc_val, exc_tb):
        close = self.content_container.button('X', kwargs={"id": "close"})
        if close:
            self.container.write('''<style>#modal {
                    display: none;
                }</style>''', unsafe_allow_html=True)
            if self.close_fn:
                self.close_fn()

            return {"action": "close"}
        return None



import sys

sys.path.append('D:\code\python\component-template\\template')

from streamlit_rightly_component import rightly_component, set_debug, init_component, modal
import streamlit as st

init_component()
set_debug()

def get_url():
    # 获取当前页面 url
    url = rightly_component(component_name='getUrl')
    return url

def render_dagre():
    col1, col2 = st.columns([2, 10])
    with col1:
        plan_info = {
            "name": "测试方案名称001",
            "start_date": "2022-04-25",
            "end_date": "2022-05-01",
            "shop_count": 4,
            "fee_cost": 123456,
            "predict_gmv": 123456,
        }
        plan_info_click = rightly_component(component_name='plan-ui/plan-info', data=plan_info)
    with col2:
        nodes = {
            'id': 'final',
            'name': 'final',
            'type': 'final',
            "value": 100,
            'children': [{
                "id": '5.pay_amt|item_id|biz_date|channel2:allchn',
                "name": "tb",
                "type": "platform",
                "value": 100,
                "children": [{
                    "id": "5.pay_amt|item_id|biz_date|channel2:allchn__190043425",
                    "name": "190043425",
                    "type": "shop",
                    "value": 100,
                    "children": [
                        {'id': '5.tb_adv_strategy__190043425','type': 'input'},
                        {'id': '5.tb_price_strategy__190043425','type': 'input'},
                        {'id': '5.pay_amt|item_id|biz_date|channel2:allchn__190043425', 'type': 'output'}
                    ]
                }]
            }]
        }

        edges = [
            {'source': '5.pay_amt|item_id|biz_date|channel2:allchn',
            'target': '5.pay_amt|item_id|biz_date|channel2:allchn__190043425'},
            {'source': '5.tb_adv_strategy__190043425',
            'target': '5.pay_amt|item_id|biz_date|channel2:allchn__190043425'},
            {'source': '5.tb_price_strategy__190043425',
            'target': '5.pay_amt|item_id|biz_date|channel2:allchn__190043425'},
            {'source': '5.pay_amt|item_id|biz_date|channel2:allchn', 'target': 'final'}
        ]

        dagre_click = rightly_component(component_name='Dag',
            data={"nodes": nodes, "edges": edges},
            default=None
        )

        if dagre_click and dagre_click["action"] == "shopClick":
            st.session_state['show_shop'] = dagre_click["data"]["node_id"]
        if dagre_click and dagre_click["action"] == "inputNodeClick" and dagre_click["value_is_change"]:
            st.session_state['open_modal'] = {
                "node": dagre_click["data"]["node_id"]
            }

        if st.session_state.get('show_shop'):
            darge_dag = rightly_component(component_name='dagre',
                data={"nodes":[{"id":"npc|item_id|biz_date|channel2:allchn__190043425","label":"客均件数"},{"id":"tb_price_strategy__190043425","label":"天猫商品定价策略"},{"id":"tb_adv_strategy__190043425","label":"天猫广告策略"},{"id":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","label":"天猫商品定价策略"},{"id":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","label":"天猫广告策略"},{"id":"ztc_upc__190043425","label":"ztcupc"},{"id":"pay_conv_rate|item_id|biz_date|channel:mtbsearch__190043425","label":"搜索支付转化率"},{"id":"mtbrecomm_uv__190043425","label":"推荐uv"},{"id":"ylmf_upc__190043425","label":"ylmfupc"},{"id":"mtbsearch_uv__190043425","label":"搜索uv"},{"id":"wxt_upc__190043425","label":"wxtupc"},{"id":"pay_conv_rate|item_id|biz_date|channel:cart__190043425","label":"购物车支付转化率"},{"id":"pay_conv_rate|item_id|biz_date|channel:mtbrecomm__190043425","label":"推荐支付转化率"},{"id":"ztc_uv__190043425","label":"ztc uv"},{"id":"ylmf_uv__190043425","label":"ylmf uv"},{"id":"wxt_uv__190043425","label":"wxt uv"},{"id":"allchn_cart_byr_cnt__190043425","label":"全渠道加购买家数"},{"id":"allchn_clt_byr_cnt__190043425","label":"全渠道加购买家数"},{"id":"cart_uv__190043425","label":"购物车引导UV"},{"id":"mytb_uv__190043425","label":"我的淘宝引导UV"},{"id":"uv|item_id|biz_date|channel2:allchn__190043425","label":"全渠道去重UV"},{"id":"pay_conv_rate|item_id|biz_date|channel2:allchn__190043425","label":"全渠道支付转化率"},{"id":"pay_conv_rate_allchn_lr__190043425","label":"全渠道去重转化率"},{"id":"pay_byr_cnt|item_id|biz_date|channel2:allchn__190043425","label":"全渠道买家数"},{"id":"pay_amt|item_id|biz_date|channel2:allchn__190043425","label":"成交金额"}],"edges":[{"source":"npc|item_id|biz_date|channel2:allchn__190043425","target":"pay_amt|item_id|biz_date|channel2:allchn__190043425"},{"source":"tb_price_strategy__190043425","target":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425"},{"source":"tb_adv_strategy__190043425","target":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ztc_upc__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:mtbsearch__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mtbrecomm_uv__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ylmf_upc__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mtbsearch_uv__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel2:allchn__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"wxt_upc__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:cart__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_amt|item_id|biz_date|channel2:allchn__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"cart_uv__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mytb_uv__190043425"},{"source":"tb_price_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:mtbrecomm__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ztc_upc__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ztc_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:mtbsearch__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mtbrecomm_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ylmf_upc__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mtbsearch_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel2:allchn__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"wxt_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"ylmf_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"wxt_upc__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:cart__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"cart_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"mytb_uv__190043425"},{"source":"tb_adv_strategy__shop_id_cate_spu_item_id_act_name_biz_date__190043425","target":"pay_conv_rate|item_id|biz_date|channel:mtbrecomm__190043425"},{"source":"ztc_upc__190043425","target":"ztc_uv__190043425"},{"source":"pay_conv_rate|item_id|biz_date|channel:mtbsearch__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"mtbrecomm_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"mtbrecomm_uv__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"mtbrecomm_uv__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"mtbrecomm_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"ylmf_upc__190043425","target":"ylmf_uv__190043425"},{"source":"mtbsearch_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"mtbsearch_uv__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"mtbsearch_uv__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"mtbsearch_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"wxt_upc__190043425","target":"wxt_uv__190043425"},{"source":"ztc_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"ztc_uv__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"ztc_uv__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"ztc_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"ylmf_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"ylmf_uv__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"ylmf_uv__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"ylmf_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"wxt_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"wxt_uv__190043425","target":"allchn_cart_byr_cnt__190043425"},{"source":"wxt_uv__190043425","target":"allchn_clt_byr_cnt__190043425"},{"source":"wxt_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"allchn_cart_byr_cnt__190043425","target":"cart_uv__190043425"},{"source":"allchn_clt_byr_cnt__190043425","target":"mytb_uv__190043425"},{"source":"cart_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"cart_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"mytb_uv__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"mytb_uv__190043425","target":"uv|item_id|biz_date|channel2:allchn__190043425"},{"source":"uv|item_id|biz_date|channel2:allchn__190043425","target":"pay_conv_rate|item_id|biz_date|channel2:allchn__190043425"},{"source":"uv|item_id|biz_date|channel2:allchn__190043425","target":"pay_conv_rate_allchn_lr__190043425"},{"source":"uv|item_id|biz_date|channel2:allchn__190043425","target":"pay_byr_cnt|item_id|biz_date|channel2:allchn__190043425"},{"source":"pay_conv_rate_allchn_lr__190043425","target":"pay_byr_cnt|item_id|biz_date|channel2:allchn__190043425"},{"source":"pay_byr_cnt|item_id|biz_date|channel2:allchn__190043425","target":"pay_amt|item_id|biz_date|channel2:allchn__190043425"}]},
                key="dagre2")
            if darge_dag and darge_dag["value_is_change"]:
                st.session_state['open_modal'] = {
                    "node": darge_dag["data"]["node_id"]
                }
        
    if st.session_state.get('open_modal', None):
        # 展示弹窗
        def close_fn():
            del st.session_state['open_modal']
            # widget_id = ctx.session_state._state._get_widget_id('dagre2')
            # ctx.session_state._state[widget_id] = None
            # print(ctx.session_state._state[widget_id])

        with modal(close_fn=close_fn) as container:
            with container:
                col1, col2 = st.columns(2)
                with col1:
                    st.write(st.session_state['open_modal']["node"])
                with col2:
                    st.write('col 2')

render_dagre()


import streamlit as st
import pandas as pd
import numpy as np

from st_aggrid import AgGrid, GridOptionsBuilder, JsCode

df = pd.DataFrame(
    "",
    index=range(10),
    columns=list("abcde"),
)

gb = GridOptionsBuilder.from_dataframe(df)
gb.configure_default_column(editable=True)

gb.configure_column('a',
    cellEditor='agRichSelectCellEditor',
    cellEditorParams={'values':['a','b','c']}
)

gb.configure_grid_options(enableRangeSelection=True)


response = AgGrid(
    df,
    gridOptions=gb.build(),
    fit_columns_on_grid_load=True,
    allow_unsafe_jscode=True,
    enable_enterprise_modules=True
)


from streamlit_rightly_component import rightly_component, set_debug, instance
import pandas as pd
import streamlit as st
from inspect import signature

MAX_SIZE = 128
CUSTOM_SESSION_CACHE = '_custom_session_cache'

def session_cache(fn):
  """ session 缓存，单次会话有效

  Args:
      fn (function): 要进行缓存的函数

  Returns:
      new_fn (function): 返回装饰后的新函数
  """
  if CUSTOM_SESSION_CACHE not in st.session_state:
    st.session_state[CUSTOM_SESSION_CACHE] = {
      "cache": {},
      "lst": []
    }

  sig = signature(fn)
  def wrapTheFunction(*args, **kwargs):
    bound_values = sig.bind(*args, **kwargs)
    key = bound_values.__str__()
    value = st.session_state[CUSTOM_SESSION_CACHE]['cache'].get(key)

    # 命中缓存
    if value is not None:
      return value

    # 最多缓存 128 个函数结果，包括不同入参
    if len(st.session_state[CUSTOM_SESSION_CACHE]['lst']) >= MAX_SIZE:
      oldkey = st.session_state[CUSTOM_SESSION_CACHE]['lst'].pop(0)
      if oldkey in st.session_state[CUSTOM_SESSION_CACHE]['cache']:
        st.session_state[CUSTOM_SESSION_CACHE]['cache'].pop(oldkey)

    result = fn(*args, **kwargs)
    st.session_state[CUSTOM_SESSION_CACHE]['lst'].append(key)
    st.session_state[CUSTOM_SESSION_CACHE]['cache'][key] = result
    return result

  return wrapTheFunction


def clear_session_cache():
  """ 清空所有缓存
  """
  st.session_state[CUSTOM_SESSION_CACHE] = {
    "cache": {},
    "lst": []
  }


@session_cache
def get_data(client_id):
  print('get_data]', client_id)
  return { "id": client_id }


if st.button('clear'):
  st.write('clear')
  # clear_session_cache()

st.write(get_data(1))

st.write(get_data(1))
st.write(get_data(2))